#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Sagemaker processors for launcing and managering ML/Deep Learning jobs."""

from sagemaker.local import LocalSession
from sagemaker.sklearn.processing import SKLearnProcessor

from ..log import get_logger
from .session import get_sagemaker_session

logger = get_logger(__name__)


def get_sklearn_processor(
    creds: dict, instance_type: str, job_name: str
) -> SKLearnProcessor:
    """Creates a sklearn processor

    Args:
        creds (str): Credentials to use for the session
        instance_type (str): The instance type to use for the amazon sagemaker notebook instance.
        job_name (str): The name of the job.

    Returns:
        SKLearnProcessor: sklearn processor object
    """

    logger.info("Getting sklearn processor")
    role_arn = creds["aws"]["roleArn"]
    logger.info(f"Role ARN: {role_arn}")
    logger.info(f"Instance type: {instance_type}")

    if instance_type == "local":
        sagemaker_session = LocalSession()
        sagemaker_session.config = {"local": {"local_code": True}}
    else:
        sagemaker_session = get_sagemaker_session(creds)

    processor = SKLearnProcessor(
        framework_version="0.20.0",
        role=role_arn,
        instance_type=instance_type,
        instance_count=1,
        base_job_name=job_name,
        sagemaker_session=sagemaker_session,
    )

    return processor



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module for handling various warehouse connections"""

from typing import Union

from .connector_base import connector_classes
from .redshift_connector import RedShiftConnector
from .snowflake_connector import SnowflakeConnector

def ProfilesConnector(config: dict, **kwargs) -> Union[RedShiftConnector, SnowflakeConnector]:
    """Creates a connector object based on the config provided

    Args:
        config: A dictionary containing the credentials and database information for the connector.
        **kwargs: Additional keyword arguments to pass to the connector.

    Returns:
        ConnectorBase: Connector object.

    Raises:
        Exception: Connector not found
    """

    warehouse_type = config.get("type").lower()
    connector = connector_classes.get(warehouse_type, None)
    if connector is None:
        raise Exception(f"Connector {warehouse_type} not found")

    aws_config = kwargs["aws_config"];  
    creds = {
        "user": config.get("user"),
        "password": config.get("password"),
        "account_identifier": config.get("account"),
        "warehouse": config.get("warehouse"),
        "host": config.get("host"),
        "port": config.get("port"),
        "s3Bucket": aws_config.get("bucket"),
        "s3SubDirectory": aws_config.get("path")
    }

    if "role" in config:
        creds["role"] = config.get("role")

    db_config = {
        "database": config.get("dbname"),
        "schema": config.get("schema")
    }

    connector = connector(creds, db_config, **kwargs)
    return connector

def ConnectorNew(config: dict, warehouse: str, **kwargs) -> Union[RedShiftConnector, SnowflakeConnector]:
    """Creates a connector object based on the config provided

    Args:
        config: A dictionary containing the credentials and database information for the connector.
        warehouse: The warehouse name to be used
        **kwargs: Additional keyword arguments to pass to the connector.

    Returns:
        ConnectorBase: Connector object.

    Raises:
        Exception: Connector not found
    """

    connector = connector_classes.get(warehouse.lower(), None)
    if connector is None:
        raise Exception(f"Connector {warehouse} not found")

    creds = {
        "user": config.get("user"),
        "password": config.get("password"),
        "account_identifier": config.get("account"),
        "warehouse": config.get("warehouse")
    }

    db_config = {
        "database": config.get("dbname"),
        "schema": config.get("schema")
    }

    connector = connector(creds, db_config, **kwargs)
    return connector

def Connector(
    config: dict, **kwargs
) -> Union[RedShiftConnector, SnowflakeConnector]:
    """Creates a connector object based on the config provided

    Args:
        config: A dictionary containing the configuration for the connector.
        **kwargs: Additional keyword arguments to pass to the connector.

    Returns:
        ConnectorBase: Connector object.

    Raises:
        Exception: No Credentials found
        Exception: Connector not found
    """

    #When using explicit warehouse name, config can be treated as credentials
    name = config.get("name").lower() if "name" in config else ""
    creds = config.get(name) if name in config else None

    if creds is None:
        raise Exception("No credentials found for {}".format(name))

    connector = connector_classes.get(name, None)
    if connector is None:
        raise Exception(f"Connector {name} not found")

    connector = connector(creds, config, **kwargs)
    return connector



"""
Data needs to be loaded from wh with the given input credentials and input features. This has helper functions to load the data
This will be used in both feature_processing and predict notebooks to load the data from warehouse.
"""

import pandas as pd

from rudderlabs.data.apps.aws.s3 import read_csv_from_s3
from rudderlabs.data.apps.wh import Connector


class DataIO:
    def __init__(self, notebook_config: dict, creds_config: dict) -> None:
        self.notebook_config = notebook_config
        self.creds_config = creds_config

        # Expect table information(name, schema, database) from data_warehouse section of
        # credentials configurations
        self.database = creds_config["data_warehouse"]["database"]
        self.schema = creds_config["data_warehouse"]["schema"]
        self.feature_store_table = creds_config["data_warehouse"][
            "feature_store_table"
        ]
        self.prediction_store_table = creds_config["data_warehouse"][
            "prediction_store_table"
        ]

    def get_warehouse_connector(self) -> Connector:
        """Returns warehouse connector

        Returns:
            Connector: Warehouse connector
        """
        warehouse_creds = self.creds_config["data_warehouse"]
        aws_config = self.creds_config["aws"]

        wh_connector = Connector(warehouse_creds, aws_config=aws_config)
        return wh_connector

    def get_data(self) -> pd.DataFrame:
        """Method for getting data from warehouse/feature store/other data sources."""

        query_string = self.notebook_config.get("query_train_data", "")
        df = pd.DataFrame()

        try:
            wh_connector = self.get_warehouse_connector()
            df = wh_connector.run_query(query_string)
        except Exception as e:
            print(str(e))

        return df

    def get_data_for_prediction(self) -> pd.DataFrame:
        """Method for getting data from warehouse/feature store/other data sources."""
        print("Getting data for prediction")
        s3_location = "s3://data-apps-sample-datasets/lead_scoring/data_for_prediction.csv"
        df = read_csv_from_s3(self.creds_config, s3_location, header=0)
        return df

    def get_data_from_s3(self, s3_location: str) -> pd.DataFrame:
        """Method for getting data from warehouse/feature store/other data sources."""
        print("Getting data from s3")
        df = read_csv_from_s3(self.creds_config, s3_location, header=0)
        return df

    def write_to_wh_table(
        self,
        df: pd.DataFrame,
        table_name: str,
        schema: str = None,
        if_exists: str = "append",
    ) -> None:
        """Writes dataframe to warehouse feature store table.

        Args:
            df (pd.DataFrame): Dataframe to be written to warehouse feature store table
            table_name (str): Feature store table name
            schema (str, optional): Schema name, Defaults to None.
            if_exists (str, optional): {"append", "replace", "fail"} Defaults to "append".
                fail: If the table already exists, the write fails.
                replace: If the table already exists, the table is dropped and the write is executed.
                append: If the table already exists, the write is executed with new rows appended to existing table
        """
        print("Writing to warehouse")
        warehouse_creds = self.creds_config["data_warehouse"]
        aws_config = self.creds_config["aws"]

        wh_connector = Connector(warehouse_creds, aws_config=aws_config)
        wh_connector.write_to_table(df, table_name, schema, if_exists)

    def update_wh_table(
        self, data: dict, table_name: str, schema: str, where: str
    ) -> None:
        """Updates warehouse table with data.

        Args:
            data (dict): Data to be updated in warehouse table
            table_name (str): Table name
            schema (str, optional): Schema name, Defaults to None.
        """
        print("Updating warehouse")
        wh_connector = self.get_warehouse_connector()
        update_values = ", ".join([f"{k} = '{v}'" for k, v in data.items()])
        sql_query = (
            f"UPDATE {schema}.{table_name} SET {update_values} WHERE {where}"
        )

        wh_connector.run_query(sql_query)
